from vcCommand import *
import vcMatrix, vcVector

cmd = getCommand()
app = cmd.Application 
sel = app.SelectionManager


def OnStart():
  global coordinates_prop, axis_prop, step_prop, count_prop, apply_prop, cancel_prop
  global current_selection
  current_selection = sel.getSelection(VC_SELECTION_COMPONENT)
  if not current_selection:
    print 'Error with Clone command. Please select a component first.'
    return
  coordinates_prop = createProperty(VC_STRING,'Coordinates', VC_PROPERTY_STEP)
  coordinates_prop.StepValues = ['World', 'Object']
  coordinates_prop.Value = 'World'
  coordinates_prop.OnChanged = visualize_current

  axis_prop = createProperty(VC_STRING,'Axis', VC_PROPERTY_STEP)
  axis_prop.StepValues = ['+X', '-X', '+Y', '-Y', '+Z', '-Z']
  axis_prop.Value = '+X'
  axis_prop.OnChanged = visualize_current

  step_prop = createProperty(VC_REAL, 'Step')
  step_prop.Quantity = 'Distance'
  step_prop.Value = 1000
  step_prop.OnChanged = visualize_current
  
  count_prop = createProperty(VC_INTEGER, 'Count')
  count_prop.Value = 3
  count_prop.OnChanged = visualize_current
  
  apply_prop = createProperty(VC_BUTTON, 'Apply')
  apply_prop.OnChanged = do_clone
  
  cancel_prop = createProperty(VC_BUTTON, 'Clear')
  cancel_prop.OnChanged = clear_bound_boxes
  
  visualize_current()
  executeInActionPanel()


def visualize_current(arg=None):
  global coordinates_prop, axis_prop, step_prop, count_prop, apply_prop
  geo_cont = app.Simulation.World.UserGeometry
  geo_cont.clear()
  for comp in current_selection:
    bd1 = comp.BoundDiagonal
    bd2 = comp.BoundDiagonal
    bd3 = comp.BoundDiagonal
    bd4 = comp.BoundDiagonal
    bd5 = comp.BoundDiagonal
    bd6 = comp.BoundDiagonal
    bd7 = comp.BoundDiagonal
    bd8 = comp.BoundDiagonal
    bd2.X = -bd1.X
    bd3.Y = -bd1.Y
    bd4.X, bd4.Y = -bd1.X, -bd1.Y
    bd5.Z = -bd1.Z
    bd6.X, bd6.Z = -bd1.X, -bd6.Z
    bd7.Y, bd7.Z = -bd1.Y, -bd1.Z
    bd8.X, bd8.Y, bd8.Z = -bd1.X, -bd1.Y, -bd1.Z
    bc = comp.BoundCenter 
    p1 = bc+bd1
    p2 = bc+bd2
    p3 = bc+bd3
    p4 = bc+bd4
    p5 = bc+bd5
    p6 = bc+bd6
    p7 = bc+bd7
    p8 = bc+bd8
    for i in range(count_prop.Value):
      counter = i
      wpr = comp.WorldPositionMatrix
      if coordinates_prop.Value == 'World':
        if axis_prop.Value == '+X': wpr.translateAbs(counter * step_prop.Value ,0 ,0)
        elif axis_prop.Value == '-X': wpr.translateAbs(-counter * step_prop.Value ,0 ,0)
        elif axis_prop.Value == '+Y': wpr.translateAbs(0, counter * step_prop.Value ,0)
        elif axis_prop.Value == '-Y': wpr.translateAbs(0, -counter * step_prop.Value ,0)
        elif axis_prop.Value == '+Z': wpr.translateAbs(0, 0, counter * step_prop.Value)
        elif axis_prop.Value == '-Z': wpr.translateAbs(0, 0, -counter * step_prop.Value)
      elif coordinates_prop.Value == 'Object':
        if axis_prop.Value == '+X': wpr.translateRel(counter * step_prop.Value ,0 ,0)
        elif axis_prop.Value == '-X': wpr.translateRel(-counter * step_prop.Value ,0 ,0)
        elif axis_prop.Value == '+Y': wpr.translateRel(0, counter * step_prop.Value ,0)
        elif axis_prop.Value == '-Y': wpr.translateRel(0, -counter * step_prop.Value ,0)
        elif axis_prop.Value == '+Z': wpr.translateRel(0, 0, counter * step_prop.Value)
        elif axis_prop.Value == '-Z': wpr.translateRel(0, 0, -counter * step_prop.Value)
      lineset = geo_cont.createGeometrySet(VC_COMPACTLINESET)
      lineset.LineWidth = 2
      lineset.Material = comp.Material
      lineset.addLine([wpr*p1, wpr*p2, wpr*p4, wpr*p3, wpr*p1])
      lineset.addLine([wpr*p5, wpr*p6, wpr*p8, wpr*p7, wpr*p5])
      lineset.addLine([wpr*p1, wpr*p5])
      lineset.addLine([wpr*p2, wpr*p6])
      lineset.addLine([wpr*p3, wpr*p7])
      lineset.addLine([wpr*p4, wpr*p8])
      lineset.update()
  app.render()


def do_clone(arg=None):
  global coordinates_prop, axis_prop, step_prop, count_prop, apply_prop
  for i in range(count_prop.Value-1):
    counter = i+1
    for comp in current_selection:
      wpr = comp.WorldPositionMatrix
      if coordinates_prop.Value == 'World':
        if axis_prop.Value == '+X': wpr.translateAbs(counter * step_prop.Value ,0 ,0)
        elif axis_prop.Value == '-X': wpr.translateAbs(-counter * step_prop.Value ,0 ,0)
        elif axis_prop.Value == '+Y': wpr.translateAbs(0, counter * step_prop.Value ,0)
        elif axis_prop.Value == '-Y': wpr.translateAbs(0, -counter * step_prop.Value ,0)
        elif axis_prop.Value == '+Z': wpr.translateAbs(0, 0, counter * step_prop.Value)
        elif axis_prop.Value == '-Z': wpr.translateAbs(0, 0, -counter * step_prop.Value)
      elif coordinates_prop.Value == 'Object':
        if axis_prop.Value == '+X': wpr.translateRel(counter * step_prop.Value ,0 ,0)
        elif axis_prop.Value == '-X': wpr.translateRel(-counter * step_prop.Value ,0 ,0)
        elif axis_prop.Value == '+Y': wpr.translateRel(0, counter * step_prop.Value ,0)
        elif axis_prop.Value == '-Y': wpr.translateRel(0, -counter * step_prop.Value ,0)
        elif axis_prop.Value == '+Z': wpr.translateRel(0, 0, counter * step_prop.Value)
        elif axis_prop.Value == '-Z': wpr.translateRel(0, 0, -counter * step_prop.Value)
      newcomp = app.cloneComponent(comp)    
      newcomp.PositionMatrix = wpr
      newcomp.update()
      newcomp.saveState()
  clear_bound_boxes()
  app.render()


def clear_bound_boxes(arg=None):
  geo_cont = app.Simulation.World.UserGeometry
  geo_cont.clear()
  app.render()


addState( None )